/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "MainFrm.h"
#include "ToolFrm.h"
#include "LCTankCircuitFrm.h"
#include "ToolSquareFrm.h"
#include "ShSimpsonsRule.h"
#include "ShInductanceCalculator.h"
#include "ShInductanceCalculator_Square.h"
//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WMSysCommand(TWMSysCommand& Msg)
{
	switch(Msg.CmdType)
	{
	case SC_MAXIMIZE:
		return;
	}

	TForm::Dispatch(&Msg);
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::WmNCLButtonDblClk(TMessage& Msg)
{
	if(Msg.WParam == HTCAPTION)
	{
		return;
	}

	TForm::Dispatch(&Msg);
}
//---------------------------------------------------------------------------
// \mCh: Pʂɍ킹C_N^X̒l𕶎ŕԂ܂B
std::wstring __fastcall TMainForm::GetInductanceText_0(double inductance) const
{
	const int miliHenry  = 0;
	const int microHenry = 1;
	const int nanoHenry  = 2;

	double retValue;

	if(this->Unit_0_ComboBox->ItemIndex == miliHenry)
	{
		retValue = inductance * 1e3;
	}
	else if(this->Unit_0_ComboBox->ItemIndex == microHenry)
	{
		retValue = inductance * 1e6;
	}
	else if(this->Unit_0_ComboBox->ItemIndex == nanoHenry)
	{
		retValue = inductance * 1e9;
	}

	return String(retValue).t_str();
}
//---------------------------------------------------------------------------
// \mCh: sb`̒l擾܂B0.0 ȂOX[܂B
double __fastcall TMainForm::GetPitch_0() const
{
	const double pitch = this->Pitch_0_Edit->Text.ToDouble() * 1e-3;
	if(ShMath::float_equals<double>(pitch, 0.0))
	{
		throw std::range_error("sb` 0 wł܂B");
	}

	return pitch;
}
//---------------------------------------------------------------------------
// `fʃRC: Pʂɍ킹C_N^X̒l𕶎ŕԂ܂B
std::wstring __fastcall TMainForm::GetInductanceText_1(double inductance) const
{
	const int miliHenry  = 0;
	const int microHenry = 1;
	const int nanoHenry  = 2;

	double retValue;

	if(this->Unit_1_ComboBox->ItemIndex == miliHenry)
	{
		retValue = inductance * 1e3;
	}
	else if(this->Unit_1_ComboBox->ItemIndex == microHenry)
	{
		retValue = inductance * 1e6;
	}
	else if(this->Unit_1_ComboBox->ItemIndex == nanoHenry)
	{
		retValue = inductance * 1e9;
	}

	return String(retValue).t_str();
}
//---------------------------------------------------------------------------
// `fʃRC: sb`̒l擾܂B0.0 ȂOX[܂B
double __fastcall TMainForm::GetPitch_1() const
{
	const double pitch = this->Pitch_1_Edit->Text.ToDouble() * 1e-3;
	if(ShMath::float_equals<double>(pitch, 0.0))
	{
		throw std::range_error("sb` 0 wł܂B");
	}

	return pitch;
}
//---------------------------------------------------------------------------
// \mCh: C_N^XvZ܂B
void __fastcall TMainForm::Calc_0_ButtonClick(TObject *Sender)
{
	try
	{
		TShInductanceCalculator<double> inductanceCalculator;

		const double radius = this->Diamiter_0_Edit->Text.ToDouble() / 2.0 * 1e-3;
		const double length = this->Length_0_Edit->Text.ToDouble() * 1e-3;

		inductanceCalculator.SetRadius(radius);
		if(this->Turns_0_RadioButton->Checked)
		{
			const double turns = this->Turns_0_Edit->Text.ToDouble();
			inductanceCalculator.SetLengthAndTurns(length, turns);
		}
		else
		{
			const double pitch = this->GetPitch_0();
			inductanceCalculator.SetLengthAndPitch(length, pitch);
		}

		this->m_inductance = inductanceCalculator.GetInductance();

		this->Inductance_0_Edit->Text
			= this->GetInductanceText_0(this->m_inductance).c_str();
	}
	catch(std::exception& exception)
	{
		const String msg = exception.what();
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg.t_str(), caption.t_str(), flags);
	}
	catch(EConvertError& exception)
	{
		const TCHAR msg[] = TEXT("͍ڂɗLȐlȊO͂Ă邩A")
			TEXT("͂ĂȂ܂B");
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg, caption.t_str(), flags);
	}
	catch(...)
	{
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::FormShow(TObject *Sender)
{
	this->PageControl1->TabIndex = 0;
	this->Turns_0_RadioButton->Checked = true;
	this->Diamiter_0_Edit->SetFocus();

	this->PageControl1->TabIndex = 1;
	this->Turns_1_RadioButton->Checked = true;
	this->Side_1_Edit->SetFocus();

	this->PageControl1->TabIndex = 0;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RadioButtons_0_Click(TObject *Sender)
{
	this->Turns_0_Edit->Enabled = this->Turns_0_RadioButton->Checked;
	this->Turns_0_Edit->Color = this->Turns_0_Edit->Enabled ? clWindow : clBtnFace;
	this->Pitch_0_Edit->Enabled = this->Pitch_0_RadioButton->Checked;
	this->Pitch_0_Edit->Color = this->Pitch_0_Edit->Enabled ? clWindow : clBtnFace;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Unit_0_ComboBoxChange(TObject *Sender)
{
	this->Inductance_0_Edit->Text
		= this->GetInductanceText_0(this->m_inductance).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::ExitItemExecute(TObject *Sender)
{
	this->Close();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetTurnFromInductanceItemExecute(TObject *Sender)
{
	ToolForm->PageControl1->TabIndex = 0;
	ToolForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetPitchFromInductanceItemExecute(TObject *Sender)
{
	ToolForm->PageControl1->TabIndex = 1;
	ToolForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetLengthFromInductanceItemExecute(TObject *Sender)
{
	ToolForm->PageControl1->TabIndex = 2;
	ToolForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetRadiusFromInductanceItemExecute(TObject *Sender)
{
	ToolForm->PageControl1->TabIndex = 3;
	ToolForm->Show();
}
//---------------------------------------------------------------------------
// `fʃRC: C_N^X̒lvZ܂B
void __fastcall TMainForm::Calc_1_ButtonClick(TObject *Sender)
{
	try
	{
		TShInductanceCalculator_Square<double> inductanceCalculator;

		const double side = this->Side_1_Edit->Text.ToDouble() * 1e-3;
		const double length = this->Length_1_Edit->Text.ToDouble() * 1e-3;

		inductanceCalculator.SetSide(side);
		if(this->Turns_1_RadioButton->Checked)
		{
			const double turns = this->Turns_1_Edit->Text.ToDouble();
			inductanceCalculator.SetLengthAndTurns(length, turns);
		}
		else
		{
			const double pitch = this->GetPitch_1();
			inductanceCalculator.SetLengthAndPitch(length, pitch);
		}

		this->m_inductance = inductanceCalculator.GetInductance();

		this->Inductance_1_Edit->Text
			= this->GetInductanceText_1(this->m_inductance).c_str();

	}
	catch(std::exception& exception)
	{
		const String msg = exception.what();
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg.t_str(), caption.t_str(), flags);
	}
	catch(EConvertError& exception)
	{
		const TCHAR msg[] = TEXT("͍ڂɗLȐlȊO͂Ă邩A")
			TEXT("͂ĂȂ܂B");
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg, caption.t_str(), flags);
	}
	catch(...)
	{
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::RadioButtons_1_Click(TObject *Sender)
{
	this->Turns_1_Edit->Enabled = this->Turns_1_RadioButton->Checked;
	this->Turns_1_Edit->Color = this->Turns_1_Edit->Enabled ? clWindow : clBtnFace;
	this->Pitch_1_Edit->Enabled = this->Pitch_1_RadioButton->Checked;
	this->Pitch_1_Edit->Color = this->Pitch_1_Edit->Enabled ? clWindow : clBtnFace;
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::Unit_1_ComboBoxChange(TObject *Sender)
{
	this->Inductance_1_Edit->Text
		= this->GetInductanceText_1(this->m_inductance).c_str();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetTurnFromInductance_SquareItemExecute(TObject *Sender)

{
	ToolSquareForm->PageControl1->TabIndex = 0;
	ToolSquareForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetPitchFromInductance_SquareItemExecute(TObject *Sender)
{
	ToolSquareForm->PageControl1->TabIndex = 1;
	ToolSquareForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetLengthFromInductance_SquareItemExecute(TObject *Sender)
{
	ToolSquareForm->PageControl1->TabIndex = 2;
	ToolSquareForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::GetSideFromInductance_SquareItemExecute(TObject *Sender)
{
	ToolSquareForm->PageControl1->TabIndex = 3;
	ToolSquareForm->Show();
}
//---------------------------------------------------------------------------
void __fastcall TMainForm::LCTankCircuitItemExecute(TObject *Sender)
{
	LCTankCircuitForm->Show();
}
//---------------------------------------------------------------------------

