/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------
// \mCh̃C_N^X߂NXłB

#ifndef ShInductanceCalculatorH
#define ShInductanceCalculatorH
//---------------------------------------------------------------------------
// RC̃C_N^X
//
// @cRCiair core inductorji~RCAL\mChj
// C_N^X L ́AW KnCʁARC̊ nARC̒fʐς SA
// RC̎̒ƂƁA
//
//     L = (Kn *  * n^2 * S) / l [H]
//
// ŕ\܂Bfʂ~̏ꍇA
//
//     S = r^2 łB
//
// @^󒆂̓ o ́A
//
//     o = 4 x 10^-7 [H/m] = [N/A^2]
//
// ŕ\܂B
// @̎AC_N^X̓RC̊ n  2 ƒfʐ S ɔႵA
// WRC̎̒ l ɔႵ܂B
//
// C_N^̓H
//
//      +---[R]---[L]---+
//      |               |
// -----+               +------
//      |               |
//      +-----[C]-------+
//
// @}́AC_N^̓HłB
// @HIɂ́AȐꍇƓɂȂ܂B
// @C_N^[ƒɓĂŔA̒RƂȂ܂B̒ŔA
// eX^[ŒRlvlƃCR[ł͂܂B́A}CNg̈ɂȂƁA
// \ʂɂāAgd̕\ʂɏW邩łB
// @ɂ́AHV[hP[XɕĂƁAC_N^[Ŕ鎥̈ꕔ
// P[Xɓ`Aɓd邱Ƃɂ鑹l܂B
//  C_N^[ƕɓĂRfTłA~RC̃^[ƂɕzI
// Ă邱ƂARC̕zeʂƌ肵Ă܂B
// @}킩悤ɁẢH͕񋤐UHł邱Ƃ킩܂BȂ킿A
// Ugȏł́ARC̓RCłȂȂ킯łB
//
// RC Q
//
// @RC QiQuality factorj́ARC̓\̂ƂďdvȂ̂łA
// ͎̌̂悤ɂȂ܂B
//
//     Q = L / R
//
// @gƃC_N^X܂ĂƂ΁ARȂقQƂ܂B
// @ŌRƂ́AŎ悤ɁA鑹̍vɂȂ܂B
//---------------------------------------------------------------------------
#include "ShMath.h"
#include "ShNagaokaCoefficient.h"

template<class T>
class TShInductanceCalculator
{
private:
	// ^̓
	const T m_MU0;

	// RC̔a
	T m_radius;

	// RC̒
	T m_length;

	// RC̊
	T m_turns;

protected:
public:
	// RXgN^
	TShInductanceCalculator();

	// fXgN^
	virtual ~TShInductanceCalculator();

	// RC̔a [m] ݒ肵܂B
	virtual void SetRadius(T radius);

	// RC̒ [m] Ɗݒ肵܂B
	// ́ASetLengthAndPitch() o֐Ƃ͔rIłB
	virtual void SetLengthAndTurns(T length, T turns);

	// RC̒ [m] Ɗ̃sb`ݒ肵܂B
	// ́ASetLengthAndTurns() o֐Ƃ͔rIłB
	virtual void SetLengthAndPitch(T length, T pitch);

	// RC̃C_N^XZo܂B
	virtual T GetInductance() const;
};

//---------------------------------------------------------------------------

// RXgN^
template<class T>
TShInductanceCalculator<T>::TShInductanceCalculator()
	: m_MU0(ShMath::MU0)
{
}

// fXgN^
template<class T>
TShInductanceCalculator<T>::~TShInductanceCalculator()
{
}

// RC̔a [m] ݒ肵܂B
template<class T>
void TShInductanceCalculator<T>::SetRadius(T radius)
{
	this->m_radius = radius;
}

// RC̒ [m] Ɗݒ肵܂B
// ́ASetLengthAndPitch() o֐Ƃ͔rIłB
template<class T>
void TShInductanceCalculator<T>::SetLengthAndTurns(T length, T turns)
{
	this->m_length = length;
	this->m_turns = turns;
}

// RC̒ [m] ƃsb` [m] ݒ肵܂B
// ́ASetTurns() o֐Ƃ͔rIłB
template<class T>
void TShInductanceCalculator<T>::SetLengthAndPitch(T length, T pitch)
{
	this->m_length = length;
	this->m_turns = length / pitch;
}

// RC̃C_N^XZo܂B
template<class T>
T TShInductanceCalculator<T>::GetInductance() const
{
	TShNagaokaCoefficient<T> nagaoka;
	nagaoka.SetCoilRadius(this->m_radius);
	nagaoka.SetCoilLength(this->m_length);

	// W
	const T kn = nagaoka.GetNagaokaCoefficient();

	// RC̒fʐ
	const T s = ShMath::PI * this->m_radius * this->m_radius;

	// C_N^X
	const T inductance
		= (kn * this->m_MU0 * this->m_turns * this->m_turns * s) / this->m_length;

	return inductance;
}

#endif
