/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------
// `fʃRC̃C_N^X߂NXłB

#ifndef ShInductanceCalculator_SquareH
#define ShInductanceCalculator_SquareH
//---------------------------------------------------------------------------
#include "ShMath.h"
#include "ShNagaokaCoefficient_Square.h"

template<class T>
class TShInductanceCalculator_Square
{
private:
	// ^̓
	const T m_MU0;

	// RC̕ӂ̒
	T m_side;

	// RC̒
	T m_length;

	// RC̊
	T m_turns;

protected:
public:
	// RXgN^łB
	TShInductanceCalculator_Square();

	// fXgN^łB
	virtual ~TShInductanceCalculator_Square();

	// RC̕ӂ̒ [m] ݒ肵܂B
	virtual void SetSide(T side);

	// RC̒ [m] Ɗݒ肵܂B
	// ́ASetLengthAndPitch() o֐Ƃ͔rIłB
	virtual void SetLengthAndTurns(T length, T turns);

	// RC̒ [m] Ɗ̃sb`ݒ肵܂B
	// ́ASetLengthAndTurns() o֐Ƃ͔rIłB
	virtual void SetLengthAndPitch(T length, T pitch);

	// RC̃C_N^XZo܂B
	virtual T GetInductance() const;
};

//---------------------------------------------------------------------------

// RXgN^łB
template<class T>
TShInductanceCalculator_Square<T>::TShInductanceCalculator_Square()
	: m_MU0(ShMath::MU0)
{
}

// fXgN^łB
template<class T>
TShInductanceCalculator_Square<T>::~TShInductanceCalculator_Square()
{
}

// RC̕ӂ̒ [m] ݒ肵܂B
template<class T>
void TShInductanceCalculator_Square<T>::SetSide(T side)
{
    this->m_side = side;
}

// RC̒ [m] Ɗݒ肵܂B
// ́ASetLengthAndPitch() o֐Ƃ͔rIłB
template<class T>
void TShInductanceCalculator_Square<T>::SetLengthAndTurns(T length, T turns)
{
	this->m_length = length;
	this->m_turns = turns;
}

// RC̒ [m] ƃsb` [m] ݒ肵܂B
// ́ASetTurns() o֐Ƃ͔rIłB
template<class T>
void TShInductanceCalculator_Square<T>::SetLengthAndPitch(T length, T pitch)
{
	this->m_length = length;
	this->m_turns = length / pitch;
}

// RC̃C_N^XZo܂B
template<class T>
T TShInductanceCalculator_Square<T>::GetInductance() const
{
	TShNagaokaCoefficient_Square<T> nagaoka;
	nagaoka.SetCoilSide(this->m_side);
	nagaoka.SetCoilLength(this->m_length);

	// W
	const T kn = nagaoka.GetNagaokaCoefficient();

	// RC̒fʐ
	const T s = this->m_side * this->m_side;

	// C_N^X
	const T inductance
		= kn * (this->m_MU0 * s * this->m_turns * this->m_turns) / this->m_length;

	return inductance;
}

#endif
