/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------
// RC̃C_N^XAaARC߂NXłB

#ifndef ShLengthCalculator_TurnsH
#define ShLengthCalculator_TurnsH
//---------------------------------------------------------------------------

#include "ShInductanceCalculator.h"
#include "ShMath.h"

template<class T>
class TShLengthCalculator_Turns
{
private:
	// C_N^X
	T m_inductance;

	// a
	T m_radius;

	// 
	T m_turns;

protected:
	// ߂RCɂȂ 0 ԂłB
	virtual T formula(T length);
public:
	// RXgN^łB
	TShLengthCalculator_Turns();

	// fXgN^łB
	virtual ~TShLengthCalculator_Turns();

	// p[^ݒ肵܂B
	virtual void SetParameter(T inductance, T radius, T turns);

	// vZsāARCԂ܂B
	virtual T Calculate() const;
};

//---------------------------------------------------------------------------

// RXgN^łB
template<class T>
TShLengthCalculator_Turns<T>::TShLengthCalculator_Turns()
{
}

// fXgN^łB
template<class T>
TShLengthCalculator_Turns<T>::~TShLengthCalculator_Turns()
{
}

// ߂RCɂȂ 0 ԂłB
template<class T>
T TShLengthCalculator_Turns<T>::formula(T length)
{
	TShInductanceCalculator<double> calculator;
	calculator.SetRadius(this->m_radius);
	calculator.SetLengthAndTurns(length, this->m_turns);

	return calculator.GetInductance() - this->m_inductance;
}

// p[^ݒ肵܂B
template<class T>
void TShLengthCalculator_Turns<T>::SetParameter(T inductance, T radius, T turns)
{
	this->m_inductance = inductance;
	this->m_radius = radius;
	this->m_turns = turns;
}

// vZsāARCԂ܂B
template<class T>
T TShLengthCalculator_Turns<T>::Calculate() const
{
	const T length1 = 1e-3 * 1e-6;  // ܂菬 0 ŏZG[oB // std::numeric_limits<T>::epsilon();
	const T length2 = 100.0;
	const int trial_max = 10000;
	int actual_try_count;

	double length = ShMath::TShBisection<double>::bisection(
		this->formula, length1, length2, trial_max, actual_try_count);

	return length;
}

#endif
