/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------
// `fʃRC̃C_N^XAaARC߂NXłB

#ifndef ShLengthCalculator_Turns_SquareH
#define ShLengthCalculator_Turns_SquareH
//---------------------------------------------------------------------------

#include "ShInductanceCalculator_Square.h"
#include "ShMath.h"

template<class T>
class TShLengthCalculator_Turns_Square
{
private:
	// C_N^X
	T m_inductance;

	// 
	T m_side;

	// 
	T m_turns;

protected:
	// ߂RCɂȂ 0 ԂłB
	virtual T formula(T length);
public:
	// RXgN^łB
	TShLengthCalculator_Turns_Square();

	// fXgN^łB
	virtual ~TShLengthCalculator_Turns_Square();

	// p[^ݒ肵܂B
	virtual void SetParameter(T inductance, T side, T turns);

	// vZsāARCԂ܂B
	virtual T Calculate() const;
};

//---------------------------------------------------------------------------

// RXgN^łB
template<class T>
TShLengthCalculator_Turns_Square<T>::TShLengthCalculator_Turns_Square()
{
}

// fXgN^łB
template<class T>
TShLengthCalculator_Turns_Square<T>::~TShLengthCalculator_Turns_Square()
{
}

// ߂RCɂȂ 0 ԂłB
template<class T>
T TShLengthCalculator_Turns_Square<T>::formula(T length)
{
	TShInductanceCalculator_Square<double> calculator;
	calculator.SetSide(this->m_side);
	calculator.SetLengthAndTurns(length, this->m_turns);

	return calculator.GetInductance() - this->m_inductance;
}

// p[^ݒ肵܂B
template<class T>
void TShLengthCalculator_Turns_Square<T>::SetParameter(T inductance, T side, T turns)
{
	this->m_inductance = inductance;
	this->m_side = side;
	this->m_turns = turns;
}

// vZsāARCԂ܂B
template<class T>
T TShLengthCalculator_Turns_Square<T>::Calculate() const
{
	const T length1 = std::numeric_limits<T>::epsilon();
	const T length2 = 100.0;
	const int trial_max = 10000;
	int actual_try_count;

	double length = ShMath::TShBisection<double>::bisection(
		this->formula, length1, length2, trial_max, actual_try_count);

	return length;
}

#endif

