/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------

#ifndef ShMathH
#define ShMathH
//---------------------------------------------------------------------------
#include <cmath>
#include <exception>
#include <limits>
#include <boost/function.hpp>

namespace ShMath
{

// ~łB
const long double PI = 3.141592653589793238462643383279502884197169399375105820974944592307816406286208998628034825342117067982148086513282306647;

// ^̓łB
const long double MU0 = 4.0 * PI * 1.0e-7;

//---------------------------------------------------------------------------

// Âl_̓lԂ֐ev[głB
template<class T> bool float_equals(T a, T b)
{
	const T epsilon = std::numeric_limits<T>::epsilon();

	return ((b - epsilon) < a) && (a < (b + epsilon));
}

//---------------------------------------------------------------------------

// 񕪖@ɂ` f(x) = 0 ̉߂NXłB
//
template<class T>
class TShBisection
{
public:
	typedef boost::function<T (T)> TShBisectionFunction;
private:
protected:
public:
	TShBisection()
	{
	}

	virtual ~TShBisection()
	{
	}

	// 񕪖@ɂ` f(x) = 0 ̉߂֐łB
	//
	// 
	//     function         : [IN]  lvZ֐ f(x)
	//     initial_x1       : [IN]  l x1
	//     initial_x2       : [IN]  l x2
	//     trial_max        : [IN]  ő厎s
	//     actual_try_count : [OUT] ۂ̎s
	// ߂l
	//     f(x) = 0 ̉B

	static T bisection(
			TShBisectionFunction function,
			const T& initial_x1,
			const T& initial_x2,
			int trial_max,
			int& actual_try_count)
	{
		T x1 = initial_x1;
		T x2 = initial_x2;

		T y1 = function(x1);
		T y2 = function(x2);

		// y1  y2 ȂG[B
		if(y1 * y2 > 0.0)
		{
			std::string msg(__FILE__);
			msg.append(": ");
			msg.append(__FUNCTION__);
			msg.append(
				": function(initial_x1) and function(initial_x2) are same sign");
			throw std::range_error(msg);
		}

		T c;
		actual_try_count = 0;

		if(float_equals(y1 * y2, 0.0))
		{
			if(float_equals(y1, 0.0))
			{
				c = x1;
			}
			else
			{
				c = x2;
			}
		}
		else
		{
			bool isExit = false;
			while(!isExit && actual_try_count >= 0)
			{
				const T epsilon = std::numeric_limits<T>::epsilon();

				isExit = true;
				actual_try_count += 1;
				c = (x1 + x2) / 2.0;
				T y0 = function(c);

				if(std::fabsl(y0) > epsilon)
				{
					if(actual_try_count <= trial_max)
					{
						if(std::fabsl(x1 - x2) > epsilon
						   && std::fabsl(x1 - x2) > epsilon * std::fabsl(x2))
						{
							isExit = false;
							if(y0 * y1 < 0.0)
							{
								x2 = c;
								y2 = y0;
							}
							else
							{
								x1 = c;
								y1 = y0;
							}
						}
					}
					else
					{
						std::string msg(__FILE__);
						msg.append(": ");
						msg.append(__FUNCTION__);
						msg.append(": Reached trial max count...");
						throw std::range_error(msg);
					}
				}
			}
		}

		return c;
	}
};

} // namespace ShMath

#endif
