/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------
// \mCh̒WZoNXłB

#ifndef ShNagaokaCoefficientH
#define ShNagaokaCoefficientH
//---------------------------------------------------------------------------
#include <cmath>
#include "ShMath.h"
#include "ShCompleteEllipticIntegral_FirstKind.h"
#include "ShCompleteEllipticIntegral_SecondKind.h"

template<class T>
class TShNagaokaCoefficient
{
private:
	// RC̔a
	T m_radius;

	// RC̒
	T m_length;

protected:
	// 튮Sȉ~ϕA튮Sȉ~ϕɕKvȒ萔 k Zo܂B
	virtual T GetK() const;
public:
	// RXgN^
	TShNagaokaCoefficient();

	// fXgN^
	virtual ~TShNagaokaCoefficient();

	// RC̔aݒ肵܂B
	virtual void SetCoilRadius(T radius);

	// RC̒ݒ肵܂B
	virtual void SetCoilLength(T length);

	// WZo܂B
	virtual T GetNagaokaCoefficient() const;
};

//---------------------------------------------------------------------------

// RXgN^
template<class T>
TShNagaokaCoefficient<T>::TShNagaokaCoefficient()
{
}

// fXgN^
template<class T>
TShNagaokaCoefficient<T>::~TShNagaokaCoefficient()
{
}

// 튮Sȉ~ϕA튮Sȉ~ϕɕKvȒ萔 k Zo܂B
template<class T>
T TShNagaokaCoefficient<T>::GetK() const
{
	return 1.0 / std::sqrtl(std::powl(this->m_length / (2.0 * this->m_radius), 2.0) + 1.0);
}

// RC̔aݒ肵܂B
template<class T>
void TShNagaokaCoefficient<T>::SetCoilRadius(T radius)
{
	this->m_radius = radius;
}

// RC̒ݒ肵܂B
template<class T>
void TShNagaokaCoefficient<T>::SetCoilLength(T length)
{
	this->m_length = length;
}

// WZo܂B
template<class T>
T TShNagaokaCoefficient<T>::GetNagaokaCoefficient() const
{
	// W
	T kn;

	if(ShMath::float_equals(this->m_radius, 0.0))
	{
		kn = 1.0;
	}
	else
	{
		const T k = this->GetK();

		// 튮Sȉ~ϕsNX
		TShCompleteEllipticIntegral_FirstKind<T> firstKind;

		// 튮Sȉ~ϕsNX
		TShCompleteEllipticIntegral_SecondKind<T> secondKind;

		const T integK = firstKind.Integrate(k);
		const T integE = secondKind.Integrate(k);

		kn = (4.0 / (3.0 * ShMath::PI * std::sqrtl(1.0 - k * k)))
			* (((1.0 - k * k) / (k * k)) * integK - ((1.0 - 2.0 * k * k) / (k * k)) * integE - k);
	}

	return kn;
}

#endif

