/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------
// `fʃRC̒WZoNXłB

#ifndef ShNagaokaCoefficient_SquareH
#define ShNagaokaCoefficient_SquareH
//---------------------------------------------------------------------------
#include <cmath>
#include "ShMath.h"

template<class T>
class TShNagaokaCoefficient_Square
{
private:
	// RC̈ӂ̒
	T m_side;

	// RC̒
	T m_length;
protected:
public:
	// RXgN^łB
	TShNagaokaCoefficient_Square();

	// fXgN^łB
	virtual ~TShNagaokaCoefficient_Square();

	// RC̈ӂ̒ݒ肵܂B
	virtual void SetCoilSide(T side);

	// RC̒ݒ肵܂B
	virtual void SetCoilLength(T length);

	// WZo܂B
	virtual T GetNagaokaCoefficient() const;
};

//---------------------------------------------------------------------------

// RXgN^łB
template<class T>
TShNagaokaCoefficient_Square<T>::TShNagaokaCoefficient_Square()
{
}

// fXgN^łB
template<class T>
TShNagaokaCoefficient_Square<T>::~TShNagaokaCoefficient_Square()
{
}

// RC̈ӂ̒ݒ肵܂B
template<class T>
void TShNagaokaCoefficient_Square<T>::SetCoilSide(T side)
{
	this->m_side = side;
}

// RC̒ݒ肵܂B
template<class T>
void TShNagaokaCoefficient_Square<T>::SetCoilLength(T length)
{
	this->m_length = length;
}

// WZo܂B
template<class T>
T TShNagaokaCoefficient_Square<T>::GetNagaokaCoefficient() const
{
	const T gamma = this->m_length / this->m_side;
	const T a = std::sqrtl(gamma * gamma + 1);
	const T b = std::sqrtl(gamma * gamma + 2);

	// W
	T kn;
	if(ShMath::float_equals(gamma, 0.0))
	{
		kn = 0.0;
	}
	else
	{
		kn = (2 / ShMath::PI)
			* (
				(gamma - 1.0 / gamma) * std::logl(a / (b + 1))
				+ gamma * std::logl((a + 1) / gamma)
				+ 2.0 * std::atanl(gamma / b)
				+ (a - b - std::logl(std::sqrtl(2.0) + 1.0)
					+ (gamma * gamma * gamma
						+ std::powl(gamma * gamma + 2.0, 3.0 / 2.0)
						- 2.0 * std::powl(gamma * gamma + 1, 3.0 / 2.0)
						+ std::sqrtl(2) - 1
					) / 3.0
				) / gamma
			);
	}

	return kn;
}

#endif

