/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------
// `fʃRC̃C_N^XAӁAsb`߂NXłB

#ifndef ShPitchCalculator_SquareH
#define ShPitchCalculator_SquareH
//---------------------------------------------------------------------------

#include "ShInductanceCalculator_Square.h"
#include "ShMath.h"

template<class T>
class TShPitchCalculator_Square
{
private:
	// C_N^X
	T m_inductance;

	// 
	T m_side;

	// 
	T m_length;

protected:
	// ߂sb`ɂȂ 0 ԂłB
	virtual T formula(T pitch);
public:
	// RXgN^łB
	TShPitchCalculator_Square();

	// fXgN^łB
	virtual ~TShPitchCalculator_Square();

	// p[^ݒ肵܂B
	virtual void SetParameter(T inductance, T side, T length);

	// vZsāAԂ܂B
	virtual T Calculate() const;
};

//---------------------------------------------------------------------------

// RXgN^łB
template<class T>
TShPitchCalculator_Square<T>::TShPitchCalculator_Square()
{
}

// fXgN^łB
template<class T>
TShPitchCalculator_Square<T>::~TShPitchCalculator_Square()
{
}

// ߂sb`ɂȂ 0 ԂłB
template<class T>
T TShPitchCalculator_Square<T>::formula(T pitch)
{
	TShInductanceCalculator_Square<double> calculator;
	calculator.SetSide(this->m_side);
	calculator.SetLengthAndPitch(this->m_length, pitch);

	return calculator.GetInductance() - this->m_inductance;
}

// p[^ݒ肵܂B
template<class T>
void TShPitchCalculator_Square<T>::SetParameter(T inductance, T side, T length)
{
	this->m_inductance = inductance;
	this->m_side = side;
	this->m_length = length;
}

// vZsāAԂ܂B
template<class T>
T TShPitchCalculator_Square<T>::Calculate() const
{
	const T pitch1 = std::numeric_limits<T>::epsilon();
	const T pitch2 = 100;
	const int trial_max = 10000;
	int actual_try_count;

	double pitch = ShMath::TShBisection<double>::bisection(
		this->formula, pitch1, pitch2, trial_max, actual_try_count);

	return pitch;
}

#endif

