/*
	Coil Master - A coil inductance calculator.
	Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	MA 02110-1301 USA
*/
//---------------------------------------------------------------------------
// RC̃C_N^XAAӂ߂NXłB

#ifndef ShSideCalculator_Turns_SquareH
#define ShSideCalculator_Turns_SquareH
//---------------------------------------------------------------------------

#include "ShInductanceCalculator_Square.h"
#include "ShMath.h"

template<class T>
class TShSideCalculator_Turns_Square
{
private:
	// C_N^X
	T m_inductance;

	// 
	T m_length;

	// 
	T m_turns;

protected:
	// ߂ӂɂȂ 0 ԂłB
	virtual T formula(T side);
public:
	// RXgN^łB
	TShSideCalculator_Turns_Square();

	// fXgN^łB
	virtual ~TShSideCalculator_Turns_Square();

	// p[^ݒ肵܂B
	virtual void SetParameter(T inductance, T length, T turns);

	// vZsāAӂԂ܂B
	virtual T Calculate() const;
};

//---------------------------------------------------------------------------

// RXgN^łB
template<class T>
TShSideCalculator_Turns_Square<T>::TShSideCalculator_Turns_Square()
{
}

// fXgN^łB
template<class T>
TShSideCalculator_Turns_Square<T>::~TShSideCalculator_Turns_Square()
{
}

// ߂ӂɂȂ 0 ԂłB
template<class T>
T TShSideCalculator_Turns_Square<T>::formula(T side)
{
	TShInductanceCalculator_Square<double> calculator;
	calculator.SetSide(side);
	calculator.SetLengthAndTurns(this->m_length, this->m_turns);

	return calculator.GetInductance() - this->m_inductance;
}

// p[^ݒ肵܂B
template<class T>
void TShSideCalculator_Turns_Square<T>::SetParameter(T inductance, T length, T turns)
{
	this->m_inductance = inductance;
	this->m_length = length;
	this->m_turns = turns;
}

// vZsāAӂԂ܂B
template<class T>
T TShSideCalculator_Turns_Square<T>::Calculate() const
{
	const T side1 = std::numeric_limits<double>::epsilon();
	const T side2 = 100.0;
	const int trial_max = 10000;
	int actual_try_count;

	double length = ShMath::TShBisection<double>::bisection(
		this->formula, side1, side2, trial_max, actual_try_count);

	return length;
}

#endif

