/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------
// Vv\ Simpson's Rule gāAlϕNXłB

#ifndef ShSimpsonsRuleH
#define ShSimpsonsRuleH
//---------------------------------------------------------------------------
#include <vector>
#include <boost/function.hpp>
#include "shexception.h"

template<class T>
class TShSimpsonsRule
{
public:
	class EShSimpsonError : public shexception
	{
	public:
		explicit EShSimpsonError(const std::string& what_arg)
			: shexception(what_arg)
		{
		}
	};
public:
	typedef boost::function<T (T)> TShIntegratee;

private:
	// ϕ֐ y = f(x)  y ̒li[ꂽxN^B
	std::vector<T> m_y;

	// ϕ֐ y = f(x) ̐ϕԂ̊ԊuB
	T m_h;

protected:
	// ϕԂ̕ 2A4A6Accł邩܂B
	virtual bool isNOk(int n);

public:
	// RXgN^
	TShSimpsonsRule();

	// fXgN^
	virtual ~TShSimpsonsRule();

	// ϕ֐ݒ肵܂B
	// 
	//     x_from : ϕJn x l
	//     x_to   : ϕI x l
	//     n      : ϕԂ̕
	virtual void SetIntegratee(TShIntegratee integratee, T x_from, T x_to, int n);

	// Vv\ɂ鐔lϕs܂B
	// ߂l
	//     Vv\ɂ鐔lϕ̒lB
	virtual T Integrate();
};


//---------------------------------------------------------------------------

// RXgN^
template<class T>
TShSimpsonsRule<T>::TShSimpsonsRule()
{
}

// fXgN^
template<class T>
TShSimpsonsRule<T>::~TShSimpsonsRule()
{
}

// ϕԂ̕ 2A4A6Accł邩܂B
template<class T>
bool TShSimpsonsRule<T>::isNOk(int n)
{
	return (n > 0) && (n % 2 == 0);
}

// ϕ֐ݒ肵܂B
// 
//     x_from : ϕJn x l
//     x_to   : ϕI x l
//     n      : ϕԂ̕
template<class T>
void TShSimpsonsRule<T>::SetIntegratee(TShIntegratee integratee, T x_from, T x_to, int n)
{
	if(!this->isNOk(n))
	{
		std::string msg(__FUNCTION__);
		msg += ": n  2A4A6AccłȂ΂Ȃ܂B";
		throw EShSimpsonError(msg);
	}
	if(!(x_from <= x_to))
	{
		std::string msg(__FUNCTION__);
		msg += ": x_from <= x_to łȂ΂Ȃ܂B";
		throw EShSimpsonError(msg);
	}

	this->m_y.clear();

	this->m_h = (x_to - x_from) / n;

	int i = 1;
	this->m_y.push_back(integratee(x_from - this->m_h * (i - 1)));
	for( ; i <= n - 1 ; i += 2)
	{
		this->m_y.push_back(integratee(x_from + this->m_h * i));
		this->m_y.push_back(integratee(x_from + this->m_h * (i + 1)));
	}
}

// Vv\ɂ鐔lϕs܂B
// ߂l
//     Vv\ɂ鐔lϕ̒lB
template<class T>
T TShSimpsonsRule<T>::Integrate()
{
	T answer = 0.0;

	for(std::vector<T>::size_type i = 1 ; i <= this->m_y.size() - 1 ; i += 2)
	{
		answer += (this->m_h / 3.0) * (this->m_y[i - 1] + 4.0 * this->m_y[i] + this->m_y[i + 1]);
	}

	return answer;
}


#endif
