/*
    Coil Master - A coil inductance calculator.
    Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
    MA 02110-1301 USA
*/
//---------------------------------------------------------------------------
// RC̃C_N^XAaA犪߂NXłB

#ifndef ShTurnsCalculatorH
#define ShTurnsCalculatorH
//---------------------------------------------------------------------------

#include "ShInductanceCalculator.h"
#include "ShMath.h"

template<class T>
class TShTurnsCalculator
{
private:
	// C_N^X
	T m_inductance;

	// a
	T m_radius;

	// 
	T m_length;

protected:
	// ߂ɂȂ 0 ԂłB
	virtual T formula(T turns);
public:
	// RXgN^łB
	TShTurnsCalculator();

	// fXgN^łB
	virtual ~TShTurnsCalculator();

	// p[^ݒ肵܂B
	virtual void SetParameter(T inductance, T radius, T length);

	// vZsāAԂ܂B
	virtual T Calculate() const;
};

//---------------------------------------------------------------------------


// RXgN^łB
template<class T>
TShTurnsCalculator<T>::TShTurnsCalculator()
{
}

// fXgN^łB
template<class T>
TShTurnsCalculator<T>::~TShTurnsCalculator()
{
}

// ߂ɂȂ 0 ԂłB
template<class T>
T TShTurnsCalculator<T>::formula(T turns)
{
	TShInductanceCalculator<double> calculator;
	calculator.SetRadius(this->m_radius);
	calculator.SetLengthAndTurns(this->m_length, turns);

	return calculator.GetInductance() - this->m_inductance;
}

// p[^ݒ肵܂B
template<class T>
void TShTurnsCalculator<T>::SetParameter(T inductance, T radius, T length)
{
	this->m_inductance = inductance;
	this->m_radius = radius;
	this->m_length = length;
}

// vZsāAԂ܂B
template<class T>
T TShTurnsCalculator<T>::Calculate() const
{
	const int turns1 = 0;
	const int turns2 = 10000;
	const int trial_max = 10000;
	int actual_try_count;

	double turns = ShMath::TShBisection<double>::bisection(
		this->formula, turns1, turns2, trial_max, actual_try_count);

	return turns;
}

#endif
