/*
	Coil Master - A coil inductance calculator.
	Copyright (C) 2010 Hiroyuki Shimada (JQ1WDN). All rights reserved.

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
	MA 02110-1301 USA
*/
//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "ToolSquareFrm.h"
#include "ShTurnsCalculator_Square.h"
#include "ShPitchCalculator_Square.h"
#include "ShLengthCalculator_Turns_Square.h"
#include "ShLengthCalculator_Pitch_Square.h"
#include "ShSideCalculator_Turns_Square.h"
#include "ShSideCalculator_Pitch_Square.h"

// 񕪖@̐lvZŁAȂoOoB
//#undef TShLengthCalculator_Turns_Square_has_bug
#define TShLengthCalculator_Turns_Square_has_bug

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TToolSquareForm *ToolSquareForm;
//---------------------------------------------------------------------------
__fastcall TToolSquareForm::TToolSquareForm(TComponent* Owner)
	: TForm(Owner)
{
}
//---------------------------------------------------------------------------
void __fastcall TToolSquareForm::WMSysCommand(TWMSysCommand& Msg)
{
	switch(Msg.CmdType)
	{
	case SC_MAXIMIZE:
		return;
	}

	TForm::Dispatch(&Msg);
}
//---------------------------------------------------------------------------
void __fastcall TToolSquareForm::WmNCLButtonDblClk(TMessage& Msg)
{
	if(Msg.WParam == HTCAPTION)
	{
		return;
	}

	TForm::Dispatch(&Msg);
}
//---------------------------------------------------------------------------
void __fastcall TToolSquareForm::FormShow(TObject *Sender)
{
#if defined(TShLengthCalculator_Turns_Square_has_bug)
	this->Turns_2_RadioButton->Enabled = false;
	this->Pitch_2_RadioButton->Checked = true;
#else
	this->Turns_2_RadioButton->Checked = true;
#endif}
	this->Turns_3_RadioButton->Checked = true;
}
//---------------------------------------------------------------------------
double __fastcall TToolSquareForm::GetInductance(
	TEdit *inductanceEdit, TComboBox *unitComboBox) const
{
	const int miliHenry  = 0;
	const int microHenry = 1;
	const int nanoHenry  = 2;

	double inductance;

	if(unitComboBox->ItemIndex == miliHenry)
	{
		inductance = inductanceEdit->Text.ToDouble() * 1e-3;
	}
	else if(unitComboBox->ItemIndex == microHenry)
	{
		inductance = inductanceEdit->Text.ToDouble() * 1e-6;
	}
	else if(unitComboBox->ItemIndex == nanoHenry)
	{
		inductance = inductanceEdit->Text.ToDouble() * 1e-9;
	}

	return inductance;
}
//---------------------------------------------------------------------------
void __fastcall TToolSquareForm::Calc_0_ButtonClick(TObject *Sender)
// ߂B
{
	try
	{
		const double inductance
			= this->GetInductance(this->Inductance_0_Edit, this->Unit_0_ComboBox);
		const double side = this->Side_0_Edit->Text.ToDouble() * 1e-3;
		const double length = this->Length_0_Edit->Text.ToDouble() * 1e-3;

		TShTurnsCalculator_Square<double> calculator;
		calculator.SetParameter(inductance, side, length);
		const double turns = calculator.Calculate();

		this->Turns_0_Edit->Text = turns;
	}
	catch(std::exception& exception)
	{
		const String msg = exception.what();
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg.t_str(), caption.t_str(), flags);
	}
	catch(EConvertError& exception)
	{
		const TCHAR msg[] = TEXT("͍ڂɗLȐlȊO͂Ă邩A")
			TEXT("͂ĂȂ܂B");
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg, caption.t_str(), flags);
	}
	catch(...)
	{
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TToolSquareForm::Calc_1_ButtonClick(TObject *Sender)
// sb`߂B
{
	try
	{
		const double inductance
			= this->GetInductance(this->Inductance_1_Edit, this->Unit_1_ComboBox);
		const double side = this->Side_1_Edit->Text.ToDouble() * 1e-3;
		const double length = this->Length_1_Edit->Text.ToDouble() * 1e-3;

		TShPitchCalculator_Square<double> calculator;
		calculator.SetParameter(inductance, side, length);
		const double pitch = calculator.Calculate();

		this->Pitch_1_Edit->Text = pitch * 1e3;
	}
	catch(std::exception& exception)
	{
		const String msg = exception.what();
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg.t_str(), caption.t_str(), flags);
	}
	catch(EConvertError& exception)
	{
		const TCHAR msg[] = TEXT("͍ڂɗLȐlȊO͂Ă邩A")
			TEXT("͂ĂȂ܂B");
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg, caption.t_str(), flags);
	}
	catch(...)
	{
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TToolSquareForm::RadioButtons_2_Click(TObject *Sender)
{
	this->Turns_2_Edit->Enabled = this->Turns_2_RadioButton->Checked;
	this->Turns_2_Edit->Color = this->Turns_2_Edit->Enabled ? clWindow : clBtnFace;
	this->Pitch_2_Edit->Enabled = this->Pitch_2_RadioButton->Checked;
	this->Pitch_2_Edit->Color = this->Pitch_2_Edit->Enabled ? clWindow : clBtnFace;
}
//---------------------------------------------------------------------------
void __fastcall TToolSquareForm::Calc_2_ButtonClick(TObject *Sender)
// ߂
{
	try
	{
		if(this->Turns_2_RadioButton->Checked)
		{
			const double inductance
				= this->GetInductance(this->Inductance_2_Edit, this->Unit_2_ComboBox);
			const double side = this->Side_2_Edit->Text.ToDouble() * 1e-3;
			const int turns = this->Turns_2_Edit->Text.ToInt();

			TShLengthCalculator_Turns_Square<double> calculator;
			calculator.SetParameter(inductance, side, turns);
			const double length = calculator.Calculate();

			this->Length_2_Edit->Text = length * 1e3;
		}
		else if(this->Pitch_2_RadioButton->Checked)
		{
			const double inductance
				= this->GetInductance(this->Inductance_2_Edit, this->Unit_2_ComboBox);
			const double side = this->Side_2_Edit->Text.ToDouble() * 1e-3;
			const double pitch = this->Pitch_2_Edit->Text.ToDouble() * 1e-3;

			TShLengthCalculator_Pitch_Square<double> calculator;
			calculator.SetParameter(inductance, side, pitch);
			const double length = calculator.Calculate();

			this->Length_2_Edit->Text = length * 1e3;
		}
	}
	catch(std::exception& exception)
	{
		const String msg = exception.what();
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg.t_str(), caption.t_str(), flags);
	}
	catch(EConvertError& exception)
	{
		const TCHAR msg[] = TEXT("͍ڂɗLȐlȊO͂Ă邩A")
			TEXT("͂ĂȂ܂B");
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg, caption.t_str(), flags);
	}
	catch(...)
	{
		throw;
	}
}
//---------------------------------------------------------------------------
void __fastcall TToolSquareForm::RadioButtons_3_Click(TObject *Sender)
{
	this->Turns_3_Edit->Enabled = this->Turns_3_RadioButton->Checked;
	this->Turns_3_Edit->Color = this->Turns_3_Edit->Enabled ? clWindow : clBtnFace;
	this->Pitch_3_Edit->Enabled = this->Pitch_3_RadioButton->Checked;
	this->Pitch_3_Edit->Color = this->Pitch_3_Edit->Enabled ? clWindow : clBtnFace;
}
//---------------------------------------------------------------------------
void __fastcall TToolSquareForm::Calc_3_ButtonClick(TObject *Sender)
// a߂B
{
	try
	{
		if(this->Turns_3_RadioButton->Checked)
		{
			const double inductance
				= this->GetInductance(this->Inductance_3_Edit, this->Unit_3_ComboBox);
			const double length = this->Length_3_Edit->Text.ToDouble() * 1e-3;
			const int turns = this->Turns_3_Edit->Text.ToInt();

			TShSideCalculator_Turns_Square<double> calculator;
			calculator.SetParameter(inductance, length, turns);
			const double side = calculator.Calculate();

			this->Side_3_Edit->Text = side * 1e3;
		}
		else if(this->Pitch_3_RadioButton->Checked)
		{
			const double inductance
				= this->GetInductance(this->Inductance_3_Edit, this->Unit_3_ComboBox);
			const double length = this->Length_3_Edit->Text.ToDouble() * 1e-3;
			const double pitch = this->Pitch_3_Edit->Text.ToDouble() * 1e-3;

			TShSideCalculator_Pitch_Square<double> calculator;
			calculator.SetParameter(inductance, length, pitch);
			const double side = calculator.Calculate();

			this->Side_3_Edit->Text = side * 1e3;
		}
	}
	catch(std::exception& exception)
	{
		const String msg = exception.what();
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg.t_str(), caption.t_str(), flags);
	}
	catch(EConvertError& exception)
	{
		const TCHAR msg[] = TEXT("͍ڂɗLȐlȊO͂Ă邩A")
			TEXT("͂ĂȂ܂B");
		const String caption = Application->Title;
		const int flags = MB_OK | MB_ICONERROR;
		Application->MessageBox(msg, caption.t_str(), flags);
	}
	catch(...)
	{
		throw;
	}
}
//---------------------------------------------------------------------------

